@@ -1016,7 +1016,7 @@ Tato verze tutorilu je, podobn jako GNU Emacs, chrnna copyrightem a
 je ena se svolenm distribuovat kopie za jistch podmnek:
 
 Copyright (C) 1985, 1996, 1998, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009, 2010  Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
    Kadmu je zarueno prvo vytvet a distribuovat pesn kopie tohoto
    dokumentu tak, jak jej obdrel, na jakmkoliv mdiu, s tm, e bude
