@@ -1,7 +1,7 @@
 ;;; sregex.el --- symbolic regular expressions
 
 ;; Copyright (C) 1997, 1998, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 ;; Author: Bob Glickstein <bobg+sregex@zanshin.com>
 ;; Maintainer: Bob Glickstein <bobg+sregex@zanshin.com>
@@ -9,10 +9,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -565,7 +563,7 @@ has one of the following forms:
   (let ((chars (make-bool-vector 256 nil))) ; Yeah, right!
     (dolist (arg args)
       (cond ((integerp arg) (aset chars arg t))
-	    ((stringp arg) (mapcar (lambda (c) (aset chars c t)) arg))
+	    ((stringp arg) (mapc (lambda (c) (aset chars c t)) arg))
 	    ((consp arg)
 	     (let ((start (car arg))
 		   (end (cdr arg)))
@@ -606,5 +604,5 @@ has one of the following forms:
 
 (provide 'sregex)
 
-;;; arch-tag: 460c1f5a-eb6e-42ec-a451-ffac78bdf492
+;; arch-tag: 460c1f5a-eb6e-42ec-a451-ffac78bdf492
 ;;; sregex.el ends here
