@@ -1,15 +1,16 @@
 ;;; em-pred.el --- argument predicates and modifiers (ala zsh)
 
-;; Copyright (C) 1999, 2000 Free Software Foundation
+;; Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+;;   2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 ;; Author: John Wiegley <johnw@gnu.org>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,21 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
-
-(provide 'em-pred)
-
-(eval-when-compile (require 'esh-maint))
-
-(defgroup eshell-pred nil
-  "This module allows for predicates to be applied to globbing
-patterns (similar to zsh), in addition to string modifiers which can
-be applied either to globbing results, variable references, or just
-ordinary strings."
-  :tag "Value modifiers and predicates"
-  :group 'eshell-module)
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -60,6 +47,17 @@ ordinary strings."
 
 ;;; Code:
 
+(eval-when-compile (require 'eshell))
+
+;;;###autoload
+(eshell-defgroup eshell-pred nil
+  "This module allows for predicates to be applied to globbing
+patterns (similar to zsh), in addition to string modifiers which can
+be applied either to globbing results, variable references, or just
+ordinary strings."
+  :tag "Value modifiers and predicates"
+  :group 'eshell-module)
+
 ;;; User Variables:
 
 (defcustom eshell-pred-load-hook '(eshell-pred-initialize)
@@ -113,7 +111,7 @@ The format of each entry is
 (put 'eshell-predicate-alist 'risky-local-variable t)
 
 (defcustom eshell-modifier-alist
-  '((?e . '(lambda (lst)
+  '((?E . '(lambda (lst)
 	     (mapcar
 	      (function
 	       (lambda (str)
@@ -186,12 +184,13 @@ OWNERSHIP:
 
 FILE ATTRIBUTES:
   l[+-]N                 +/-/= N links
-  a[Mwhm][+-](N|'FILE')  access time +/-/= N mnths/weeks/days/mins
-			 if FILE specified, use as comparison basis;
-			 so a+'file.c' shows files accessed before
-			 file.c was last accessed
-  m[Mwhm][+-](N|'FILE')  modification time...
-  c[Mwhm][+-](N|'FILE')  change time...
+  a[Mwhms][+-](N|'FILE') access time +/-/= N mnths/weeks/hours/mins/secs
+			 (days if unspecified) if FILE specified,
+			 use as comparison basis; so a+'file.c'
+			 shows files accessed before file.c was
+			 last accessed
+  m[Mwhms][+-](N|'FILE') modification time...
+  c[Mwhms][+-](N|'FILE') change time...
   L[kmp][+-]N            file size +/-/= N Kb/Mb/blocks
 
 EXAMPLES:
@@ -207,7 +206,7 @@ EXAMPLES:
   "Eshell modifier quick reference:
 
 FOR SINGLE ARGUMENTS, or each argument of a list of strings:
-  e  evaluate again
+  E  evaluate again
   L  lowercase
   U  uppercase
   C  capitalize
@@ -255,7 +254,6 @@ EXAMPLES:
 
 (defun eshell-pred-initialize ()
   "Initialize the predicate/modifier code."
-  (make-local-hook 'eshell-parse-argument-hook)
   (add-hook 'eshell-parse-argument-hook
 	    'eshell-parse-arg-modifier t t)
   (define-key eshell-command-map [(meta ?q)] 'eshell-display-predicate-help)
@@ -410,7 +408,7 @@ returning the resultant string."
   "Return a predicate to test whether a file matches a certain time."
   (let* ((quantum 86400)
 	 qual amount when open close end)
-    (when (memq (char-after) '(?M ?w ?h ?m))
+    (when (memq (char-after) '(?M ?w ?h ?m ?s))
       (setq quantum (char-after))
       (cond
        ((eq quantum ?M)
@@ -601,4 +599,11 @@ that 'ls -l' will show in the first column of its display. "
 	 (lambda (str)
 	   (split-string str ,sep))) lst))))
 
+(provide 'em-pred)
+
+;; Local Variables:
+;; generated-autoload-file: "esh-groups.el"
+;; End:
+
+;; arch-tag: 8b5ce022-17f3-4c40-93c7-5faafaa63f31
 ;;; em-pred.el ends here
