@@ -1,7 +1,7 @@
 ;;; format.el --- read and save files in multiple formats
 
-;; Copyright (C) 1994, 1995, 1997, 1999, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997, 1999, 2001, 2002, 2003, 2004, 2005,
+;;   2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 ;; Author: Boris Goldowsky <boris@gnu.org>
 
@@ -64,35 +64,37 @@
 (put 'buffer-auto-save-file-format 'permanent-local t)
 
 (defvar format-alist
-  '((text/enriched "Extended MIME text/enriched format."
-		   "Content-[Tt]ype:[ \t]*text/enriched"
+  ;; FIXME: maybe each item can be purecopied instead of just the strings.
+  `((text/enriched ,(purecopy "Extended MIME text/enriched format.")
+		   ,(purecopy "Content-[Tt]ype:[ \t]*text/enriched")
 		   enriched-decode enriched-encode t enriched-mode)
-    (plain "ISO 8859-1 standard format, no text properties."
+    (plain ,(purecopy "ISO 8859-1 standard format, no text properties.")
 	   ;; Plain only exists so that there is an obvious neutral choice in
 	   ;; the completion list.
 	   nil nil nil nil nil)
-    (TeX   "TeX (encoding)"
+    (TeX   ,(purecopy "TeX (encoding)")
 	   nil
 	   iso-tex2iso iso-iso2tex t nil)
-    (gtex  "German TeX (encoding)"
+    (gtex  ,(purecopy "German TeX (encoding)")
 	   nil
 	   iso-gtex2iso iso-iso2gtex t nil)
-    (html  "HTML/SGML \"ISO 8879:1986//ENTITIES Added Latin 1//EN\" (encoding)"
+    (html  ,(purecopy "HTML/SGML \"ISO 8879:1986//ENTITIES Added Latin 1//EN\" (encoding)")
 	   nil
 	   iso-sgml2iso iso-iso2sgml t nil)
-    (rot13 "rot13"
+    (rot13 ,(purecopy "rot13")
 	   nil
-	   "tr a-mn-z n-za-m" "tr a-mn-z n-za-m" t nil)
-    (duden "Duden Ersatzdarstellung"
+	   ,(purecopy "tr a-mn-z n-za-m") ,(purecopy "tr a-mn-z n-za-m") t nil)
+    (duden ,(purecopy "Duden Ersatzdarstellung")
 	   nil
-	   "diac" iso-iso2duden t nil)
-    (de646 "German ASCII (ISO 646)"
+	   ,(purecopy "diac") iso-iso2duden t nil)
+    (de646 ,(purecopy "German ASCII (ISO 646)")
 	   nil
-	   "recode -f iso646-ge:latin1" "recode -f latin1:iso646-ge" t nil)
-    (denet "net German"
+	   ,(purecopy "recode -f iso646-ge:latin1") 
+	   ,(purecopy "recode -f latin1:iso646-ge") t nil)
+    (denet ,(purecopy "net German")
 	   nil
 	   iso-german iso-cvt-read-only t nil)
-    (esnet "net Spanish"
+    (esnet ,(purecopy "net Spanish")
 	   nil
 	   iso-spanish iso-cvt-read-only t nil))
   "List of information about understood file formats.
@@ -106,9 +108,9 @@ DOC-STR should be a single line providing more information about the
         the user if they ask for more information.
 
 REGEXP  is a regular expression to match against the beginning of the file;
-        it should match only files in that format.  Use nil to avoid
-        matching at all for formats for which it isn't appropriate to
-        require explicit encoding/decoding.
+        it should match only files in that format.  REGEXP may be nil, in
+        which case the format will never be applied automatically to a file.
+        Use this for formats that you only ever want to apply manually.
 
 FROM-FN is called to decode files in that format; it takes two args, BEGIN
         and END, and can make any modifications it likes, returning the new
@@ -136,6 +138,8 @@ MODE-FN, if specified, is called when visiting a file with that format.
 
 PRESERVE, if non-nil, means that `format-write-file' should not remove
           this format from `buffer-file-format'.")
+;;;###autoload
+(put 'format-alist 'risky-local-variable t)
 
 ;;; Basic Functions (called from Lisp)
 
@@ -176,8 +180,7 @@ it should be a Lisp function.  Decoding is done for the given BUFFER."
 	;; We should perhaps go via a temporary buffer and copy it
 	;; back, in case of errors.
 	(if (and (zerop (save-window-excursion
-			  (shell-command-on-region (point-min) (point-max)
-						   method t t
+			  (shell-command-on-region from to method t t
 						   error-buff)))
 		 ;; gzip gives zero exit status with bad args, for instance.
 		 (zerop (with-current-buffer error-buff
@@ -218,9 +221,6 @@ For most purposes, consider using `format-encode-region' instead."
 		  (multibyte enable-multibyte-characters)
 		  (coding-system buffer-file-coding-system))
 	      (with-current-buffer copy-buf
-		(set (make-local-variable
-		      'write-region-post-annotation-function)
-		     'kill-buffer)
 		(setq selective-display sel-disp)
 		(set-buffer-multibyte multibyte)
 		(setq buffer-file-coding-system coding-system))
@@ -228,6 +228,15 @@ For most purposes, consider using `format-encode-region' instead."
 	      (set-buffer copy-buf)
 	      (format-insert-annotations write-region-annotations-so-far from)
 	      (format-encode-run-method to-fn (point-min) (point-max) orig-buf)
+              (when (buffer-live-p copy-buf)
+                (with-current-buffer copy-buf
+                  ;; Set write-region-post-annotation-function to
+                  ;; delete the buffer once the write is done, but do
+                  ;; it after running to-fn so it doesn't affect
+                  ;; write-region calls in to-fn.
+                  (set (make-local-variable
+                        'write-region-post-annotation-function)
+                       'kill-buffer)))
 	      nil)
 	  ;; Otherwise just call function, it will return annotations.
 	  (funcall to-fn from to orig-buf)))))
