@@ -1,6 +1,7 @@
 ;;; reporter.el --- customizable bug reporting of lisp programs
 
-;; Copyright (C) 1993,1994,1995,1996,1997,1998 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 2001, 2002, 2003,
+;;   2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 ;; Author:          1993-1998 Barry A. Warsaw
 ;; Maintainer:      FSF
@@ -9,10 +10,10 @@
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +21,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-;; Boston, MA 02111-1307, USA.
+;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -117,7 +116,7 @@ composed.")
   "Periodically output a status message."
   (if (zerop (% reporter-status-count 10))
       (progn
-	(message reporter-status-message)
+	(message "%s" reporter-status-message)
 	(setq reporter-status-message (concat reporter-status-message "."))))
   (setq reporter-status-count (1+ reporter-status-count)))
 
@@ -167,13 +166,15 @@ composed.")
     (goto-char (1+ (nth 1 state)))
     (current-column)))
 
+(declare-function mail-position-on-field "sendmail" (field &optional soft))
+(declare-function mail-text "sendmail" ())
+
 (defun reporter-dump-variable (varsym mailbuf)
   "Pretty-print the value of the variable in symbol VARSYM.
 MAILBUF is the mail buffer being composed."
   (reporter-update-status)
   (condition-case nil
-      (let ((val (save-excursion
-		   (set-buffer reporter-eval-buffer)
+      (let ((val (with-current-buffer reporter-eval-buffer
 		   (symbol-value varsym)))
 	    (sym (symbol-name varsym))
 	    (print-escape-newlines t)
@@ -199,8 +200,7 @@ MAILBUF is the mail buffer being composed."
 		(reporter-beautify-list maxwidth compact-p))))
 	(insert "\n"))
     (void-variable
-     (save-excursion
-       (set-buffer mailbuf)
+     (with-current-buffer mailbuf
        (mail-position-on-field "X-Reporter-Void-Vars-Found")
        (end-of-line)
        (insert (symbol-name varsym) " ")))
@@ -245,13 +245,12 @@ dumped."
       (condition-case fault
 	  (let ((mailbuf (current-buffer))
 		(elbuf (get-buffer-create " *tmp-reporter-buffer*")))
-	    (save-excursion
-	      (set-buffer elbuf)
+	    (with-current-buffer elbuf
 	      (emacs-lisp-mode)
 	      (erase-buffer)
 	      (insert "(setq\n")
 	      (lisp-indent-line)
-	      (mapcar
+	      (mapc
 	       (function
 		(lambda (varsym-or-cons-cell)
 		  (let ((varsym (or (car-safe varsym-or-cons-cell)
@@ -263,7 +262,7 @@ dumped."
 	       varlist)
 	      (lisp-indent-line)
 	      (insert ")\n"))
-	    (insert-buffer elbuf))
+	    (insert-buffer-substring elbuf))
 	(error
 	 (insert "State could not be dumped due to the following error:\n\n"
 		 (format "%s" fault)
@@ -407,4 +406,6 @@ mail-sending package is used for editing and sending the message."
 
 
 (provide 'reporter)
+
+;; arch-tag: 33612ff4-fbbc-4be2-b183-560ce9e0199b
 ;;; reporter.el ends here
