@@ -1,7 +1,7 @@
 ;;; ldap.el --- client interface to LDAP for Emacs
 
 ;; Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
+;;   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 ;; Author: Oscar Figueiredo <oscar@cpe.fr>
 ;; Maintainer: FSF
@@ -524,8 +524,7 @@ an alist of attribute/value pairs."
 	    (equal "" filter))
 	(error "No search filter"))
     (setq filter (cons filter attributes))
-    (save-excursion
-      (set-buffer buf)
+    (with-current-buffer buf
       (erase-buffer)
       (if (and host
 	       (not (equal "" host)))
@@ -559,7 +558,8 @@ an alist of attribute/value pairs."
 	  (setq arglist (nconc arglist (list (format "-z%s" sizelimit)))))
       (eval `(call-process ldap-ldapsearch-prog
 			   nil
-			   buf
+			   ;; Ignore stderr, which can corrupt results
+			   (list buf nil)
 			   nil
 			   ,@arglist
 			   ,@ldap-ldapsearch-args
@@ -597,8 +597,7 @@ an alist of attribute/value pairs."
 	    ;; Do not try to open non-existent files
 	    (if (equal value "")
 		(setq value " ")
-	      (save-excursion
-		(set-buffer bufval)
+	      (with-current-buffer bufval
 		(erase-buffer)
 		(set-buffer-multibyte nil)
 		(insert-file-contents-literally value)
@@ -607,9 +606,9 @@ an alist of attribute/value pairs."
 	    (setq record (cons (list name value)
 			       record))
 	    (forward-line 1))
-	  (setq result (cons (if withdn
-				 (cons dn (nreverse record))
-			       (nreverse record)) result))
+	  (push (if withdn
+		    (cons dn (nreverse record))
+		  (nreverse record)) result)
 	  (setq record nil)
 	  (skip-chars-forward " \t\n")
 	  (message "Parsing results... %d" numres)
