@@ -1,7 +1,7 @@
 ;;; repeat.el --- convenient way to repeat the previous command
 
 ;; Copyright (C) 1998, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
+;;   2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 ;; Author: Will Mengarini <seldon@eskimo.com>
 ;; Created: Mo 02 Mar 98
@@ -124,7 +124,9 @@ if `repeat' is bound to C-x z, typing C-x z z z repeats the previous command
 only occurs if the final character by which `repeat' was invoked is a
 member of that sequence.  If this variable is nil, no re-execution occurs."
   :group 'convenience
-  :type 'boolean)
+  :type '(choice (const :tag "Repeat for all keys" t)
+		 (const :tag "Don't repeat" nil)
+		 (sexp :tag "Repeat for specific keys")))
 
 ;;;;; ****************** HACKS TO THE REST OF EMACS ******************* ;;;;;
 
