@@ -1,7 +1,7 @@
 /* Defines some widget utility functions.
 Copyright (C) 1992 Lucid, Inc.
 Copyright (C) 1994, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-  2010, 2011 Free Software Foundation, Inc.
+  2010, 2011, 2012 Free Software Foundation, Inc.
 
 This file is part of the Lucid Widget Library.
 
