@@ -1,6 +1,6 @@
 /* Simple client interface to DDE servers.
    Copyright (C) 1998, 2001, 2002, 2003, 2004, 2005,
-      2006, 2007, 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+      2006, 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
