@@ -4,7 +4,7 @@
 
 /*
 Copyright (C) 1993, 1994, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-  2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+  2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
