@@ -1,13 +1,14 @@
 /* Header file for the buffer manipulation primitives.
-   Copyright (C) 1985, 86, 93, 94, 95, 97, 1998, 1999, 2000, 2001
-   Free Software Foundation, Inc.
+   Copyright (C) 1985, 1986, 1993, 1994, 1995, 1997, 1998, 1999, 2000, 2001,
+                 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
+                 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,9 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 
 /* Accessing the parameters of the current buffer.  */
@@ -25,53 +24,53 @@ Boston, MA 02111-1307, USA.  */
 /* These macros come in pairs, one for the char position
    and one for the byte position.  */
 
-/* Position of beginning of buffer.  */ 
+/* Position of beginning of buffer.  */
 #define BEG (1)
-#define BEG_BYTE (1)
+#define BEG_BYTE (BEG)
 
-/* Position of beginning of accessible range of buffer.  */ 
+/* Position of beginning of accessible range of buffer.  */
 #define BEGV (current_buffer->begv)
 #define BEGV_BYTE (current_buffer->begv_byte)
 
 /* Position of point in buffer.  The "+ 0" makes this
-   not an l-value, so you can't assign to it.  Use SET_PT instead.  */ 
+   not an l-value, so you can't assign to it.  Use SET_PT instead.  */
 #define PT (current_buffer->pt + 0)
 #define PT_BYTE (current_buffer->pt_byte + 0)
 
-/* Position of gap in buffer.  */ 
+/* Position of gap in buffer.  */
 #define GPT (current_buffer->text->gpt)
 #define GPT_BYTE (current_buffer->text->gpt_byte)
 
-/* Position of end of accessible range of buffer.  */ 
+/* Position of end of accessible range of buffer.  */
 #define ZV (current_buffer->zv)
 #define ZV_BYTE (current_buffer->zv_byte)
 
-/* Position of end of buffer.  */ 
+/* Position of end of buffer.  */
 #define Z (current_buffer->text->z)
 #define Z_BYTE (current_buffer->text->z_byte)
 
 /* Macros for the addresses of places in the buffer.  */
 
-/* Address of beginning of buffer.  */ 
+/* Address of beginning of buffer.  */
 #define BEG_ADDR (current_buffer->text->beg)
 
-/* Address of beginning of accessible range of buffer.  */ 
+/* Address of beginning of accessible range of buffer.  */
 #define BEGV_ADDR (BYTE_POS_ADDR (current_buffer->begv_byte))
 
-/* Address of point in buffer.  */ 
+/* Address of point in buffer.  */
 #define PT_ADDR (BYTE_POS_ADDR (current_buffer->pt_byte))
 
-/* Address of beginning of gap in buffer.  */ 
-#define GPT_ADDR (current_buffer->text->beg + current_buffer->text->gpt_byte - 1)
+/* Address of beginning of gap in buffer.  */
+#define GPT_ADDR (current_buffer->text->beg + current_buffer->text->gpt_byte - BEG_BYTE)
 
 /* Address of end of gap in buffer.  */
-#define GAP_END_ADDR (current_buffer->text->beg + current_buffer->text->gpt_byte + current_buffer->text->gap_size - 1)
+#define GAP_END_ADDR (current_buffer->text->beg + current_buffer->text->gpt_byte + current_buffer->text->gap_size - BEG_BYTE)
 
-/* Address of end of accessible range of buffer.  */ 
+/* Address of end of accessible range of buffer.  */
 #define ZV_ADDR (BYTE_POS_ADDR (current_buffer->zv_byte))
 
-/* Address of end of buffer.  */ 
-#define Z_ADDR (current_buffer->text->beg + current_buffer->text->gap_size + current_buffer->text->z_byte - 1)
+/* Address of end of buffer.  */
+#define Z_ADDR (current_buffer->text->beg + current_buffer->text->gap_size + current_buffer->text->z_byte - BEG_BYTE)
 
 /* Size of gap.  */
 #define GAP_SIZE (current_buffer->text->gap_size)
@@ -82,6 +81,9 @@ Boston, MA 02111-1307, USA.  */
 /* Modification count.  */
 #define MODIFF (current_buffer->text->modiff)
 
+/* Character modification count.  */
+#define CHARS_MODIFF (current_buffer->text->chars_modiff)
+
 /* Overlay modification count.  */
 #define OVERLAY_MODIFF (current_buffer->text->overlay_modiff)
 
@@ -101,27 +103,51 @@ Boston, MA 02111-1307, USA.  */
 /* Similar macros to operate on a specified buffer.
    Note that many of these evaluate the buffer argument more than once.  */
 
-/* Position of beginning of buffer.  */ 
-#define BUF_BEG(buf) (1)
-#define BUF_BEG_BYTE(buf) (1)
+/* Position of beginning of buffer.  */
+#define BUF_BEG(buf) (BEG)
+#define BUF_BEG_BYTE(buf) (BEG_BYTE)
 
-/* Position of beginning of accessible range of buffer.  */ 
-#define BUF_BEGV(buf) ((buf)->begv)
-#define BUF_BEGV_BYTE(buf) ((buf)->begv_byte)
+/* The BUF_BEGV[_BYTE], BUF_ZV[_BYTE], and BUF_PT[_BYTE] macros cannot
+   be used for assignment; use SET_BUF_* macros below for that.  */
 
-/* Position of point in buffer.  */ 
-#define BUF_PT(buf) ((buf)->pt)
-#define BUF_PT_BYTE(buf) ((buf)->pt_byte)
+/* Position of beginning of accessible range of buffer.  */
+#define BUF_BEGV(buf)					\
+   (buf == current_buffer ? BEGV			\
+    : NILP (buf->begv_marker) ? buf->begv		\
+    : marker_position (buf->begv_marker))
 
-/* Position of gap in buffer.  */ 
+#define BUF_BEGV_BYTE(buf)				\
+   (buf == current_buffer ? BEGV_BYTE			\
+    : NILP (buf->begv_marker) ? buf->begv_byte		\
+    : marker_byte_position (buf->begv_marker))
+
+/* Position of point in buffer.  */
+#define BUF_PT(buf)					\
+   (buf == current_buffer ? PT				\
+    : NILP (buf->pt_marker) ? buf->pt			\
+    : marker_position (buf->pt_marker))
+
+#define BUF_PT_BYTE(buf)				\
+   (buf == current_buffer ? PT_BYTE			\
+    : NILP (buf->pt_marker) ? buf->pt_byte		\
+    : marker_byte_position (buf->pt_marker))
+
+/* Position of end of accessible range of buffer.  */
+#define BUF_ZV(buf)					\
+   (buf == current_buffer ? ZV				\
+    : NILP (buf->zv_marker) ? buf->zv			\
+    : marker_position (buf->zv_marker))
+
+#define BUF_ZV_BYTE(buf)				\
+   (buf == current_buffer ? ZV_BYTE			\
+    : NILP (buf->zv_marker) ? buf->zv_byte		\
+    : marker_byte_position (buf->zv_marker))
+
+/* Position of gap in buffer.  */
 #define BUF_GPT(buf) ((buf)->text->gpt)
 #define BUF_GPT_BYTE(buf) ((buf)->text->gpt_byte)
 
-/* Position of end of accessible range of buffer.  */ 
-#define BUF_ZV(buf) ((buf)->zv)
-#define BUF_ZV_BYTE(buf) ((buf)->zv_byte)
-
-/* Position of end of buffer.  */ 
+/* Position of end of buffer.  */
 #define BUF_Z(buf) ((buf)->text->z)
 #define BUF_Z_BYTE(buf) ((buf)->text->z_byte)
 
@@ -129,13 +155,13 @@ Boston, MA 02111-1307, USA.  */
 #define BUF_BEG_ADDR(buf) ((buf)->text->beg)
 
 /* Address of beginning of gap of buffer.  */
-#define BUF_GPT_ADDR(buf) ((buf)->text->beg + (buf)->text->gpt_byte - 1)
+#define BUF_GPT_ADDR(buf) ((buf)->text->beg + (buf)->text->gpt_byte - BEG_BYTE)
 
 /* Address of end of buffer.  */
-#define BUF_Z_ADDR(buf) ((buf)->text->beg + (buf)->text->gap_size + (buf)->text->z_byte - 1)
+#define BUF_Z_ADDR(buf) ((buf)->text->beg + (buf)->text->gap_size + (buf)->text->z_byte - BEG_BYTE)
 
 /* Address of end of gap in buffer.  */
-#define BUF_GAP_END_ADDR(buf) ((buf)->text->beg + (buf)->text->gpt_byte + (buf)->text->gap_size - 1)
+#define BUF_GAP_END_ADDR(buf) ((buf)->text->beg + (buf)->text->gpt_byte + (buf)->text->gap_size - BEG_BYTE)
 
 /* Size of gap.  */
 #define BUF_GAP_SIZE(buf) ((buf)->text->gap_size)
@@ -147,12 +173,19 @@ Boston, MA 02111-1307, USA.  */
 /* Modification count.  */
 #define BUF_MODIFF(buf) ((buf)->text->modiff)
 
+/* Character modification count.  */
+#define BUF_CHARS_MODIFF(buf) ((buf)->text->chars_modiff)
+
 /* Modification count as of last visit or save.  */
 #define BUF_SAVE_MODIFF(buf) ((buf)->text->save_modiff)
 
 /* Overlay modification count.  */
 #define BUF_OVERLAY_MODIFF(buf) ((buf)->text->overlay_modiff)
 
+/* Modification count as of last auto-save.  */
+/* FIXME: should we move this into ->text->auto_save_modiff?  */
+#define BUF_AUTOSAVE_MODIFF(buf) ((buf)->auto_save_modified)
+
 /* Interval tree of buffer.  */
 #define BUF_INTERVALS(buf) ((buf)->text->intervals)
 
@@ -197,34 +230,30 @@ Boston, MA 02111-1307, USA.  */
 	}								\
     }									\
   while (0)
-     
-
-/* Macros to set PT in the current buffer, or another buffer..  */
 
-#define SET_PT(position) (set_point (current_buffer, (position)))
+
+/* Macros to set PT in the current buffer, or another buffer.  */
+
+#define SET_PT(position) (set_point (position))
 #define TEMP_SET_PT(position) (temp_set_point (current_buffer, (position)))
 
-#define SET_PT_BOTH(position, byte) \
-  (set_point_both (current_buffer, (position), (byte)))
+#define SET_PT_BOTH(position, byte) (set_point_both (position, byte))
 #define TEMP_SET_PT_BOTH(position, byte) \
   (temp_set_point_both (current_buffer, (position), (byte)))
 
-#define BUF_SET_PT(buffer, position) \
-  (set_point ((buffer), (position)))
 #define BUF_TEMP_SET_PT(buffer, position) \
   (temp_set_point ((buffer), (position)))
 
-extern void set_point P_ ((struct buffer *, int));
-extern INLINE void temp_set_point P_ ((struct buffer *, int));
-extern void set_point_both P_ ((struct buffer *, int, int));
-extern INLINE void temp_set_point_both P_ ((struct buffer *, int, int));
-extern void enlarge_buffer_text P_ ((struct buffer *, int));
+extern void set_point P_ ((EMACS_INT));
+extern INLINE void temp_set_point P_ ((struct buffer *, EMACS_INT));
+extern void set_point_both P_ ((EMACS_INT, EMACS_INT));
+extern INLINE void temp_set_point_both P_ ((struct buffer *,
+					    EMACS_INT, EMACS_INT));
+extern void enlarge_buffer_text P_ ((struct buffer *, EMACS_INT));
 
 
 /* Macros for setting the BEGV, ZV or PT of a given buffer.
 
-   SET_BUF_PT* seet to be redundant.  Get rid of them?
-
    The ..._BOTH macros take both a charpos and a bytepos,
    which must correspond to each other.
 
@@ -259,7 +288,7 @@ extern void enlarge_buffer_text P_ ((struct buffer *, int));
    and store the charpos in CHARPOS and the bytepos in BYTEPOS.  */
 
 #define DECODE_POSITION(charpos, bytepos, pos)			\
-if (1)								\
+do								\
   {								\
     Lisp_Object __pos = (pos);					\
     if (NUMBERP (__pos))					\
@@ -275,19 +304,19 @@ if (1)								\
     else							\
       wrong_type_argument (Qinteger_or_marker_p, __pos);	\
   }								\
-else
+while (0)
 
 /* Return the address of byte position N in current buffer.  */
 
 #define BYTE_POS_ADDR(n) \
-  (((n) >= GPT_BYTE ? GAP_SIZE : 0) + (n) + BEG_ADDR - 1)
+  (((n) >= GPT_BYTE ? GAP_SIZE : 0) + (n) + BEG_ADDR - BEG_BYTE)
 
 /* Return the address of char position N.  */
 
 #define CHAR_POS_ADDR(n)			\
   (((n) >= GPT ? GAP_SIZE : 0)			\
    + buf_charpos_to_bytepos (current_buffer, n)	\
-   + BEG_ADDR - 1)
+   + BEG_ADDR - BEG_BYTE)
 
 /* Convert a character position to a byte position.  */
 
@@ -303,8 +332,8 @@ else
 
 #define PTR_BYTE_POS(ptr) \
 ((ptr) - (current_buffer)->text->beg					    \
- - (ptr - (current_buffer)->text->beg < (unsigned) GPT_BYTE ? 0 : GAP_SIZE) \
- + 1)
+ - (ptr - (current_buffer)->text->beg <= (unsigned) (GPT_BYTE - BEG_BYTE) ? 0 : GAP_SIZE) \
+ + BEG_BYTE)
 
 /* Return character at position POS.  */
 
@@ -319,7 +348,6 @@ else
 
 /* Variables used locally in FETCH_MULTIBYTE_CHAR.  */
 extern unsigned char *_fetch_multibyte_char_p;
-extern int _fetch_multibyte_char_len;
 
 /* Return character code of multi-byte form at position POS.  If POS
    doesn't point the head of valid multi-byte form, only the byte at
@@ -327,27 +355,35 @@ extern int _fetch_multibyte_char_len;
 
 #define FETCH_MULTIBYTE_CHAR(pos)				 	\
   (_fetch_multibyte_char_p = (((pos) >= GPT_BYTE ? GAP_SIZE : 0) 	\
-			       + (pos) + BEG_ADDR - 1),		 	\
-   _fetch_multibyte_char_len						\
-      = ((pos) >= GPT_BYTE ? ZV_BYTE : GPT_BYTE) - (pos),		\
-   STRING_CHAR (_fetch_multibyte_char_p, _fetch_multibyte_char_len))
+			       + (pos) + BEG_ADDR - BEG_BYTE),	 	\
+   STRING_CHAR (_fetch_multibyte_char_p))
+
+/* Return character at position POS.  If the current buffer is unibyte
+   and the character is not ASCII, make the returning character
+   multibyte.  */
+
+#define FETCH_CHAR_AS_MULTIBYTE(pos)			\
+  (!NILP (current_buffer->enable_multibyte_characters)	\
+   ? FETCH_MULTIBYTE_CHAR ((pos))			\
+   : UNIBYTE_TO_CHAR (FETCH_BYTE ((pos))))
+
 
 /* Macros for accessing a character or byte,
    or converting between byte positions and addresses,
    in a specified buffer.  */
 
-/* Return the address of character at byte position POS in buffer BUF. 
+/* Return the address of character at byte position POS in buffer BUF.
    Note that both arguments can be computed more than once.  */
 
 #define BUF_BYTE_ADDRESS(buf, pos) \
-((buf)->text->beg + (pos) - 1		\
+((buf)->text->beg + (pos) - BEG_BYTE		\
  + ((pos) >= (buf)->text->gpt_byte ? (buf)->text->gap_size : 0))
 
-/* Return the address of character at char position POS in buffer BUF. 
+/* Return the address of character at char position POS in buffer BUF.
    Note that both arguments can be computed more than once.  */
 
 #define BUF_CHAR_ADDRESS(buf, pos) \
-((buf)->text->beg + buf_charpos_to_bytepos ((buf), (pos)) - 1	\
+((buf)->text->beg + buf_charpos_to_bytepos ((buf), (pos)) - BEG_BYTE	\
  + ((pos) >= (buf)->text->gpt ? (buf)->text->gap_size : 0))
 
 /* Convert PTR, the address of a char in buffer BUF,
@@ -355,9 +391,9 @@ extern int _fetch_multibyte_char_len;
 
 #define BUF_PTR_BYTE_POS(buf, ptr)				\
 ((ptr) - (buf)->text->beg					\
- - (ptr - (buf)->text->beg < (unsigned) BUF_GPT_BYTE ((buf))	\
+ - (ptr - (buf)->text->beg <= (unsigned) (BUF_GPT_BYTE ((buf)) - BEG_BYTE)\
     ? 0 : BUF_GAP_SIZE ((buf)))					\
- + 1)
+ + BEG_BYTE)
 
 /* Return the character at byte position POS in buffer BUF.   */
 
@@ -378,11 +414,8 @@ extern int _fetch_multibyte_char_len;
 #define BUF_FETCH_MULTIBYTE_CHAR(buf, pos)				\
   (_fetch_multibyte_char_p						\
      = (((pos) >= BUF_GPT_BYTE (buf) ? BUF_GAP_SIZE (buf) : 0)		\
-        + (pos) + BUF_BEG_ADDR (buf) - 1),				\
-   _fetch_multibyte_char_len						\
-     = (((pos) >= BUF_GPT_BYTE (buf) ? BUF_ZV_BYTE (buf) : BUF_GPT_BYTE (buf)) \
-        - (pos)),							\
-   STRING_CHAR (_fetch_multibyte_char_p, _fetch_multibyte_char_len))
+        + (pos) + BUF_BEG_ADDR (buf) - BEG_BYTE),			\
+   STRING_CHAR (_fetch_multibyte_char_p))
 
 /* Define the actual buffer data structures.  */
 
@@ -396,26 +429,30 @@ struct buffer_text
        e.g. happen when malloc is called.  So, don't pass a pointer
        into a buffer's text to functions that malloc.  */
     unsigned char *beg;
-    
-    int gpt;			/* Char pos of gap in buffer.  */
-    int z;			/* Char pos of end of buffer.  */
-    int gpt_byte;		/* Byte pos of gap in buffer.  */
-    int z_byte;			/* Byte pos of end of buffer.  */
-    int gap_size;		/* Size of buffer's gap.  */
+
+    EMACS_INT gpt;		/* Char pos of gap in buffer.  */
+    EMACS_INT z;		/* Char pos of end of buffer.  */
+    EMACS_INT gpt_byte;		/* Byte pos of gap in buffer.  */
+    EMACS_INT z_byte;		/* Byte pos of end of buffer.  */
+    EMACS_INT gap_size;		/* Size of buffer's gap.  */
     int modiff;			/* This counts buffer-modification events
 				   for this buffer.  It is incremented for
 				   each such event, and never otherwise
 				   changed.  */
+    int chars_modiff;           /* This is modified with character change
+				   events for this buffer.  It is set to
+				   modiff for each such event, and never
+				   otherwise changed.  */
     int save_modiff;		/* Previous value of modiff, as of last
 				   time buffer visited or saved a file.  */
 
     int overlay_modiff;		/* Counts modifications to overlays.  */
 
     /* Minimum value of GPT - BEG since last redisplay that finished.  */
-    int beg_unchanged;
+    EMACS_INT beg_unchanged;
 
     /* Minimum value of Z - GPT since last redisplay that finished.  */
-    int end_unchanged;
+    EMACS_INT end_unchanged;
 
     /* MODIFF as of last redisplay that finished; if it matches MODIFF,
        beg_unchanged and end_unchanged contain no useful information.  */
@@ -433,7 +470,12 @@ struct buffer_text
        This is actually a single marker ---
        successive elements in its marker `chain'
        are the other markers referring to this buffer.  */
-    Lisp_Object markers;
+    struct Lisp_Marker *markers;
+
+    /* Usually 0.  Temporarily set to 1 in decode_coding_gap to
+       prevent Fgarbage_collect from shrinking the gap and loosing
+       not-yet-decoded bytes.  */
+    int inhibit_shrinking;
   };
 
 /* This is the structure that the buffer Lisp object points to.  */
@@ -445,14 +487,13 @@ struct buffer
 
      Check out mark_buffer (alloc.c) to see why.  */
 
-  EMACS_INT size;
-
-  /* Next buffer, in chain of all buffers including killed buffers.
+  /* HEADER.NEXT is the next buffer, in chain of all buffers,
+     including killed buffers.
      This chain is used only for garbage collection, in order to
      collect killed buffers properly.
      Note that vectors and most pseudovectors are all on one chain,
      but buffers are on a separate chain of their own.  */
-  struct buffer *next;
+  struct vectorlike_header header;
 
   /* This structure holds the coordinates of the buffer contents
      in ordinary buffers.  In indirect buffers, this is not used.  */
@@ -464,17 +505,17 @@ struct buffer
   struct buffer_text *text;
 
   /* Char position of point in buffer.  */
-  int pt;
+  EMACS_INT pt;
   /* Byte position of point in buffer.  */
-  int pt_byte;
+  EMACS_INT pt_byte;
   /* Char position of beginning of accessible range.  */
-  int begv;
+  EMACS_INT begv;
   /* Byte position of beginning of accessible range.  */
-  int begv_byte;
+  EMACS_INT begv_byte;
   /* Char position of end of accessible range.  */
-  int zv;
+  EMACS_INT zv;
   /* Byte position of end of accessible range.  */
-  int zv_byte;
+  EMACS_INT zv_byte;
 
   /* In an indirect buffer, this points to the base buffer.
      In an ordinary buffer, it is 0.  */
@@ -487,20 +528,23 @@ struct buffer
      this means the variable is always local in all buffers.  */
 #define MAX_PER_BUFFER_VARS 50
   char local_flags[MAX_PER_BUFFER_VARS];
-    
+
   /* Set to the modtime of the visited file when read or written.
      -1 means visited file was nonexistent.
      0 means visited file modtime unknown; in no case complain
      about any mismatch on next save attempt.  */
   int modtime;
-  /* the value of text->modiff at the last auto-save.  */
+  /* The value of text->modiff at the last auto-save.  */
   int auto_save_modified;
+  /* The value of text->modiff at the last display error.
+     Redisplay of this buffer is inhibited until it changes again.  */
+  int display_error_modiff;
   /* The time at which we detected a failure to auto-save,
      Or -1 if we didn't have a failure.  */
   int auto_save_failure_time;
   /* Position in buffer at which display started
      the last time this buffer was displayed.  */
-  int last_window_start;
+  EMACS_INT last_window_start;
 
   /* Set nonzero whenever the narrowing is changed in this buffer.  */
   int clip_changed;
@@ -528,6 +572,21 @@ struct buffer
      displaying this buffer.  */
   unsigned prevent_redisplay_optimizations_p : 1;
 
+  /* List of overlays that end at or before the current center,
+     in order of end-position.  */
+  struct Lisp_Overlay *overlays_before;
+
+  /* List of overlays that end after  the current center,
+     in order of start-position.  */
+  struct Lisp_Overlay *overlays_after;
+
+  /* Position where the overlay lists are centered.  */
+  EMACS_INT overlay_center;
+
+  /* Everything from here down must be a Lisp_Object.  */
+  /* buffer-local Lisp variables start at `undo_list',
+     tho only the ones from `name' on are GC'd normally.  */
+
   /* Changes in the buffer are recorded here for undo.
      t means don't record anything.
      This information belongs to the base buffer of an indirect buffer,
@@ -537,8 +596,6 @@ struct buffer
      This comes before `name' because it is marked in a special way.  */
   Lisp_Object undo_list;
 
-  /* Everything from here down must be a Lisp_Object */
-
   /* The name of this buffer.  */
   Lisp_Object name;
 
@@ -546,12 +603,15 @@ struct buffer
   Lisp_Object filename;
   /* Dir for expanding relative file names.  */
   Lisp_Object directory;
-  /* True iff this buffer has been backed up (if you write to the
+  /* True if this buffer has been backed up (if you write to the
      visited file and it hasn't been backed up, then a backup will
      be made).  */
   /* This isn't really used by the C code, so could be deleted.  */
   Lisp_Object backed_up;
   /* Length of file when last read or saved.
+     -1 means auto saving turned off because buffer shrank a lot.
+     -2 means don't turn off auto saving if buffer shrinks.
+       (That value is used with buffer-swap-text.)
      This is not in the  struct buffer_text
      because it's not used in indirect buffers at all.  */
   Lisp_Object save_length;
@@ -566,8 +626,9 @@ struct buffer
      point into this buffer or may point nowhere.  */
   Lisp_Object mark;
 
-  /* Alist of elements (SYMBOL . VALUE-IN-THIS-BUFFER)
-     for all per-buffer variables of this buffer.  */
+  /* Alist of elements (SYMBOL . VALUE-IN-THIS-BUFFER) for all
+     per-buffer variables of this buffer.  For locally unbound
+     symbols, just the symbol appears as the element.  */
   Lisp_Object local_var_alist;
 
   /* Symbol naming major mode (eg, lisp-mode).  */
@@ -576,7 +637,7 @@ struct buffer
   Lisp_Object mode_name;
   /* Mode line element that controls format of mode line.  */
   Lisp_Object mode_line_format;
-    
+
   /* Analogous to mode_line_format for the line displayed at the top
      of windows.  Nil means don't display that line.  */
   Lisp_Object header_line_format;
@@ -590,9 +651,9 @@ struct buffer
   /* This buffer's category table.  */
   Lisp_Object category_table;
 
-  /* Values of several buffer-local variables */
+  /* Values of several buffer-local variables.  */
   /* tab-width is buffer-local so that redisplay can find it
-     in buffers that are not current */
+     in buffers that are not current.  */
   Lisp_Object case_fold_search;
   Lisp_Object tab_width;
   Lisp_Object fill_column;
@@ -618,6 +679,8 @@ struct buffer
 
   /* Non-nil means do not display continuation lines.  */
   Lisp_Object truncate_lines;
+  /* Non-nil means to use word wrapping when displaying continuation lines.  */
+  Lisp_Object word_wrap;
   /* Non-nil means display ctl chars with uparrow.  */
   Lisp_Object ctl_arrow;
   /* Non-nil means display text from right to left.  */
@@ -641,17 +704,6 @@ struct buffer
   /* t means the mark and region are currently active.  */
   Lisp_Object mark_active;
 
-  /* List of overlays that end at or before the current center,
-     in order of end-position.  */
-  Lisp_Object overlays_before;
-
-  /* List of overlays that end after  the current center,
-     in order of start-position.  */
-  Lisp_Object overlays_after;
-
-  /* Position where the overlay lists are centered.  */
-  Lisp_Object overlay_center;
-
   /* Non-nil means the buffer contents are regarded as multi-byte
      form of characters, not a binary code.  */
   Lisp_Object enable_multibyte_characters;
@@ -660,9 +712,12 @@ struct buffer
      saving.  */
   Lisp_Object buffer_file_coding_system;
 
-  /* List of symbols naming the file format used for visited file. */
+  /* List of symbols naming the file format used for visited file.  */
   Lisp_Object file_format;
 
+  /* List of symbols naming the file format used for auto-save file.  */
+  Lisp_Object auto_save_file_format;
+
   /* True if the newline position cache and width run cache are
      enabled.  See search.c and indent.c.  */
   Lisp_Object cache_long_line_scans;
@@ -709,15 +764,36 @@ struct buffer
 
   /* Incremented each time the buffer is displayed in a window.  */
   Lisp_Object display_count;
- 
+
   /* Widths of left and right marginal areas for windows displaying
      this buffer.  */
-  Lisp_Object left_margin_width, right_margin_width;
+  Lisp_Object left_margin_cols, right_margin_cols;
+
+  /* Widths of left and right fringe areas for windows displaying
+     this buffer.  */
+  Lisp_Object left_fringe_width, right_fringe_width;
+
+  /* Non-nil means fringes are drawn outside display margins;
+     othersize draw them between margin areas and text.  */
+  Lisp_Object fringes_outside_margins;
+
+  /* Width and type of scroll bar areas for windows displaying
+     this buffer.  */
+  Lisp_Object scroll_bar_width, vertical_scroll_bar_type;
 
   /* Non-nil means indicate lines not displaying text (in a style
      like vi).  */
   Lisp_Object indicate_empty_lines;
 
+  /* Non-nil means indicate buffer boundaries and scrolling.  */
+  Lisp_Object indicate_buffer_boundaries;
+
+  /* Logical to physical fringe bitmap mappings.  */
+  Lisp_Object fringe_indicator_alist;
+
+  /* Logical to physical cursor bitmap mappings.  */
+  Lisp_Object fringe_cursor_alist;
+
   /* Time stamp updated each time this buffer is displayed in a window.  */
   Lisp_Object display_time;
 
@@ -726,7 +802,7 @@ struct buffer
      that point ends up this number of lines from the top of the
      window.  Nil means that scrolling method isn't used.  */
   Lisp_Object scroll_up_aggressively;
-    
+
   /* If scrolling the display because point is above the top of a
      window showing this buffer, try to choose a window start so
      that point ends up this number of lines from the bottom of the
@@ -740,6 +816,11 @@ struct buffer
   /* An integer > 0 means put that number of pixels below text lines
      in the display of this buffer.  */
   Lisp_Object extra_line_spacing;
+
+  /* *Cursor type to display in non-selected windows.
+     t means to use hollow box cursor.
+     See `cursor-type' for other values.  */
+  Lisp_Object cursor_in_non_selected_windows;
 };
 
 
@@ -776,38 +857,52 @@ extern struct buffer buffer_local_flags;
    that don't have such names.  */
 
 extern struct buffer buffer_local_symbols;
-
-/* This structure holds the required types for the values in the
-   buffer-local slots.  If a slot contains Qnil, then the
-   corresponding buffer slot may contain a value of any type.  If a
-   slot contains an integer, then prospective values' tags must be
-   equal to that integer (except nil is always allowed).
-   When a tag does not match, the function
-   buffer_slot_type_mismatch will signal an error.
-
-   If a slot here contains -1, the corresponding variable is read-only.  */
-
-extern struct buffer buffer_local_types;
 
+extern void delete_all_overlays P_ ((struct buffer *));
 extern void reset_buffer P_ ((struct buffer *));
-extern void evaporate_overlays P_ ((int));
-extern int overlays_at P_ ((int, int, Lisp_Object **, int *, int *, int *, int));
+extern void evaporate_overlays P_ ((EMACS_INT));
+extern int overlays_at P_ ((EMACS_INT pos, int extend, Lisp_Object **vec_ptr,
+			    int *len_ptr, EMACS_INT *next_ptr,
+			    EMACS_INT *prev_ptr, int change_req));
 extern int sort_overlays P_ ((Lisp_Object *, int, struct window *));
-extern void recenter_overlay_lists P_ ((struct buffer *, int));
-extern int overlay_strings P_ ((int, struct window *, unsigned char **));
+extern void recenter_overlay_lists P_ ((struct buffer *, EMACS_INT));
+extern int overlay_strings P_ ((EMACS_INT, struct window *, unsigned char **));
 extern void validate_region P_ ((Lisp_Object *, Lisp_Object *));
 extern void set_buffer_internal P_ ((struct buffer *));
 extern void set_buffer_internal_1 P_ ((struct buffer *));
 extern void set_buffer_temp P_ ((struct buffer *));
 extern void record_buffer P_ ((Lisp_Object));
-extern void buffer_slot_type_mismatch P_ ((int));
-extern void fix_overlays_before P_ ((struct buffer *, int, int));
+extern void buffer_slot_type_mismatch P_ ((Lisp_Object, int)) NO_RETURN;
+extern void fix_overlays_before P_ ((struct buffer *, EMACS_INT, EMACS_INT));
 extern void mmap_set_vars P_ ((int));
 
+/* Get overlays at POSN into array OVERLAYS with NOVERLAYS elements.
+   If NEXTP is non-NULL, return next overlay there.
+   See overlay_at arg CHANGE_REQ for meaning of CHRQ arg.  */
+
+#define GET_OVERLAYS_AT(posn, overlays, noverlays, nextp, chrq)		\
+  do {									\
+    int maxlen = 40;							\
+    overlays = (Lisp_Object *) alloca (maxlen * sizeof (Lisp_Object));	\
+    noverlays = overlays_at (posn, 0, &overlays, &maxlen,		\
+			     nextp, NULL, chrq);				\
+    if (noverlays > maxlen)						\
+      {									\
+	maxlen = noverlays;						\
+	overlays = (Lisp_Object *) alloca (maxlen * sizeof (Lisp_Object)); \
+	noverlays = overlays_at (posn, 0, &overlays, &maxlen,		\
+				 nextp, NULL, chrq);			\
+      }									\
+  } while (0)
+
+EXFUN (Fbuffer_list, 1);
+EXFUN (Fbuffer_live_p, 1);
 EXFUN (Fbuffer_name, 1);
 EXFUN (Fget_file_buffer, 1);
 EXFUN (Fnext_overlay_change, 1);
 EXFUN (Fdelete_overlay, 1);
+EXFUN (Fbuffer_local_value, 2);
+EXFUN (Fgenerate_new_buffer_name, 2);
 
 /* Functions to call before and after each text change.  */
 extern Lisp_Object Vbefore_change_functions;
@@ -845,7 +940,7 @@ extern Lisp_Object Vtransient_mark_mode;
    We assume you know which buffer it's pointing into.  */
 
 #define OVERLAY_POSITION(P) \
- (GC_MARKERP (P) ? marker_position (P) : (abort (), 0))
+ (MARKERP (P) ? marker_position (P) : (abort (), 0))
 
 
 /***********************************************************************
@@ -889,8 +984,26 @@ extern int last_per_buffer_idx;
        (B)->local_flags[IDX] = (VAL);			\
      } while (0)
 
-/* Return the index of the per-buffer variable at offset OFFSET in the
-   buffer structure.  */
+/* Return the index value of the per-buffer variable at offset OFFSET
+   in the buffer structure.
+
+   If the slot OFFSET has a corresponding default value in
+   buffer_defaults, the index value is positive and has only one
+   nonzero bit.  When a buffer has its own local value for a slot, the
+   bit for that slot (found in the same slot in this structure) is
+   turned on in the buffer's local_flags array.
+
+   If the index value is -1, even though there may be a
+   DEFVAR_PER_BUFFER for the slot, there is no default value for it;
+   and the corresponding slot in buffer_defaults is not used.
+
+   If the index value is -2, then there is no DEFVAR_PER_BUFFER for
+   the slot, but there is a default value which is copied into each
+   new buffer.
+
+   If a slot in this structure corresponding to a DEFVAR_PER_BUFFER is
+   zero, that is a bug */
+
 
 #define PER_BUFFER_IDX(OFFSET) \
       XINT (*(Lisp_Object *)((OFFSET) + (char *) &buffer_local_flags))
@@ -913,8 +1026,5 @@ extern int last_per_buffer_idx;
 #define PER_BUFFER_SYMBOL(OFFSET) \
       (*(Lisp_Object *)((OFFSET) + (char *) &buffer_local_symbols))
 
-/* Return the type of the per-buffer variable at offset OFFSET in the
-   buffer structure.  */
-
-#define PER_BUFFER_TYPE(OFFSET) \
-      (*(Lisp_Object *)((OFFSET) + (char *) &buffer_local_types))
+/* arch-tag: 679305dd-d41c-4a50-b170-3caf5c97b2d1
+   (do not change this comment) */
