@@ -1,13 +1,14 @@
 /* MS-DOS specific Lisp utilities.  Coded by Manabu Higashida, 1991.
    Major changes May-July 1993 Morten Welinder (only 10% original code left)
-   Copyright (C) 1991, 1993, 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1991, 1993, 1996, 1997, 1998, 2001, 2002, 2003, 2004,
+                 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
-GNU Emacs is free software; you can redistribute it and/or modify
+GNU Emacs is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 GNU Emacs is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +16,7 @@ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with GNU Emacs; see the file COPYING.  If not, write to
-the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-Boston, MA 02111-1307, USA.  */
-
+along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -26,46 +24,56 @@ Boston, MA 02111-1307, USA.  */
 /* The entire file is within this conditional */
 
 #include <stdio.h>
+#include <string.h>
 #include <dos.h>
+#include <setjmp.h>
 #include "lisp.h"
 #include "buffer.h"
 #include "termchar.h"
-#include "termhooks.h"
 #include "frame.h"
+#include "termhooks.h"
 #include "blockinput.h"
 #include "window.h"
 #include "dosfns.h"
 #include "msdos.h"
+#include "dispextern.h"
+#include "character.h"
+#include "coding.h"
+#include "process.h"
 #include <dpmi.h>
 #include <go32.h>
 #include <dirent.h>
+#include <sys/vfs.h>
+#include <unistd.h>
+#include <grp.h>
+#include <crt0.h>
 
 #ifndef __DJGPP_MINOR__
 # define __tb _go32_info_block.linear_address_of_transfer_buffer;
 #endif
 
 DEFUN ("int86", Fint86, Sint86, 2, 2, 0,
-  "Call specific MSDOS interrupt number INTERRUPT with REGISTERS.\n\
-Return the updated REGISTER vector.\n\
-\n\
-INTERRUPT should be an integer in the range 0 to 255.\n\
-REGISTERS should be a vector produced by `make-register' and\n\
-`set-register-value'.")
-  (interrupt, registers)
-  Lisp_Object interrupt, registers;
+       doc: /* Call specific MS-DOS interrupt number INTERRUPT with REGISTERS.
+Return the updated REGISTER vector.
+
+INTERRUPT should be an integer in the range 0 to 255.
+REGISTERS should be a vector produced by `make-register' and
+`set-register-value'.  */)
+     (interrupt, registers)
+     Lisp_Object interrupt, registers;
 {
   register int i;
   int no;
   union REGS inregs, outregs;
   Lisp_Object val;
 
-  CHECK_NUMBER (interrupt, 0);
+  CHECK_NUMBER (interrupt);
   no = (unsigned long) XINT (interrupt);
-  CHECK_VECTOR (registers, 1);
-  if (no < 0 || no > 0xff || XVECTOR (registers)-> size != 8) 
+  CHECK_VECTOR (registers);
+  if (no < 0 || no > 0xff || XVECTOR (registers)-> size != 8)
     return Qnil;
   for (i = 0; i < 8; i++)
-    CHECK_NUMBER (XVECTOR (registers)->contents[i], 1);
+    CHECK_NUMBER (XVECTOR (registers)->contents[i]);
 
   inregs.x.ax    = (unsigned long) XFASTINT (XVECTOR (registers)->contents[0]);
   inregs.x.bx    = (unsigned long) XFASTINT (XVECTOR (registers)->contents[1]);
@@ -91,25 +99,25 @@ REGISTERS should be a vector produced by `make-register' and\n\
 }
 
 DEFUN ("msdos-memget", Fdos_memget, Sdos_memget, 2, 2, 0,
-  "Read DOS memory at offset ADDRESS into VECTOR.\n\
-Return the updated VECTOR.")
-  (address, vector)
-  Lisp_Object address, vector;
+       doc: /* Read DOS memory at offset ADDRESS into VECTOR.
+Return the updated VECTOR.  */)
+     (address, vector)
+     Lisp_Object address, vector;
 {
   register int i;
   int offs, len;
   char *buf;
   Lisp_Object val;
 
-  CHECK_NUMBER (address, 0);
+  CHECK_NUMBER (address);
   offs = (unsigned long) XINT (address);
-  CHECK_VECTOR (vector, 1);
+  CHECK_VECTOR (vector);
   len = XVECTOR (vector)-> size;
-  if (len < 1 || len > 2048 || address < 0 || address > 0xfffff - len) 
+  if (len < 1 || len > 2048 || offs < 0 || offs > 0xfffff - len)
     return Qnil;
   buf = alloca (len);
   dosmemget (offs, len, buf);
-  
+
   for (i = 0; i < len; i++)
     XVECTOR (vector)->contents[i] = make_number (buf[i]);
 
@@ -117,26 +125,26 @@ Return the updated VECTOR.")
 }
 
 DEFUN ("msdos-memput", Fdos_memput, Sdos_memput, 2, 2, 0,
-  "Write DOS memory at offset ADDRESS from VECTOR.")
-  (address, vector)
-  Lisp_Object address, vector;
+       doc: /* Write DOS memory at offset ADDRESS from VECTOR.  */)
+     (address, vector)
+     Lisp_Object address, vector;
 {
   register int i;
   int offs, len;
   char *buf;
   Lisp_Object val;
 
-  CHECK_NUMBER (address, 0);
+  CHECK_NUMBER (address);
   offs = (unsigned long) XINT (address);
-  CHECK_VECTOR (vector, 1);
+  CHECK_VECTOR (vector);
   len = XVECTOR (vector)-> size;
-  if (len < 1 || len > 2048 || address < 0 || address > 0xfffff - len) 
+  if (len < 1 || len > 2048 || offs < 0 || offs > 0xfffff - len)
     return Qnil;
   buf = alloca (len);
 
   for (i = 0; i < len; i++)
     {
-      CHECK_NUMBER (XVECTOR (vector)->contents[i], 1);
+      CHECK_NUMBER (XVECTOR (vector)->contents[i]);
       buf[i] = (unsigned char) XFASTINT (XVECTOR (vector)->contents[i]) & 0xFF;
     }
 
@@ -145,14 +153,14 @@ DEFUN ("msdos-memput", Fdos_memput, Sdos_memput, 2, 2, 0,
 }
 
 DEFUN ("msdos-set-keyboard", Fmsdos_set_keyboard, Smsdos_set_keyboard, 1, 2, 0,
-  "Set keyboard layout according to COUNTRY-CODE.\n\
-If the optional argument ALLKEYS is non-nil, the keyboard is mapped for\n\
-all keys; otherwise it is only used when the ALT key is pressed.\n\
-The current keyboard layout is available in dos-keyboard-code.")
-  (country_code, allkeys)
-  Lisp_Object country_code;
+       doc: /* Set keyboard layout according to COUNTRY-CODE.
+If the optional argument ALLKEYS is non-nil, the keyboard is mapped for
+all keys; otherwise it is only used when the ALT key is pressed.
+The current keyboard layout is available in dos-keyboard-code.  */)
+     (country_code, allkeys)
+     Lisp_Object country_code, allkeys;
 {
-  CHECK_NUMBER (country_code, 0);
+  CHECK_NUMBER (country_code);
   if (!dos_set_keyboard (XINT (country_code), !NILP (allkeys)))
     return Qnil;
   return Qt;
@@ -162,8 +170,8 @@ The current keyboard layout is available in dos-keyboard-code.")
 /* Later we might want to control the mouse interface with this function,
    e.g., with respect to non-80 column screen modes.  */
 
-DEFUN ("msdos-mouse-p", Fmsdos_mouse_p, Smsdos_mouse_p, 0, 0, 0, "\
-Report whether a mouse is present.")
+DEFUN ("msdos-mouse-p", Fmsdos_mouse_p, Smsdos_mouse_p, 0, 0, 0,
+       doc: /* Report whether a mouse is present.  */)
      ()
 {
   if (have_mouse)
@@ -171,17 +179,11 @@ Report whether a mouse is present.")
   else
     return Qnil;
 }
-
-/* Function to translate colour names to integers.  See lisp/term/pc-win.el
-   for its definition.  */
-
-Lisp_Object Qmsdos_color_translate;
 #endif
 
-
 DEFUN ("msdos-mouse-init", Fmsdos_mouse_init, Smsdos_mouse_init, 0, 0, "",
-  "Initialize and enable mouse if available.")
-  ()
+       doc: /* Initialize and enable mouse if available.  */)
+     ()
 {
   if (have_mouse)
     {
@@ -193,8 +195,8 @@ DEFUN ("msdos-mouse-init", Fmsdos_mouse_init, Smsdos_mouse_init, 0, 0, "",
 }
 
 DEFUN ("msdos-mouse-enable", Fmsdos_mouse_enable, Smsdos_mouse_enable, 0, 0, "",
-  "Enable mouse if available.")
-  ()
+       doc: /* Enable mouse if available.  */)
+     ()
 {
   if (have_mouse)
     {
@@ -205,26 +207,25 @@ DEFUN ("msdos-mouse-enable", Fmsdos_mouse_enable, Smsdos_mouse_enable, 0, 0, "",
 }
 
 DEFUN ("msdos-mouse-disable", Fmsdos_mouse_disable, Smsdos_mouse_disable, 0, 0, "",
-  "Disable mouse if available.")
-  ()
+       doc: /* Disable mouse if available.  */)
+     ()
 {
   mouse_off ();
   if (have_mouse) have_mouse = -1;
   return Qnil;
 }
 
-DEFUN ("insert-startup-screen", Finsert_startup_screen, Sinsert_startup_screen, 0, 0, "", "\
-Insert copy of screen contents prior to starting emacs.\n\
-Return nil if startup screen is not available.")
-  ()
+DEFUN ("insert-startup-screen", Finsert_startup_screen, Sinsert_startup_screen, 0, 0, "",
+       doc: /* Insert copy of screen contents prior to starting Emacs.
+Return nil if startup screen is not available.  */)
+     ()
 {
   char *s;
-  int rows, cols;
-  int i, j;
-  
+  int rows, cols, i, j;
+
   if (!dos_get_saved_screen (&s, &rows, &cols))
     return Qnil;
-  
+
   for (i = 0; i < rows; i++)
     {
       for (j = 0; j < cols; j++)
@@ -239,11 +240,11 @@ Return nil if startup screen is not available.")
 }
 
 /* country info */
-int dos_country_code;
-int dos_codepage;
-int dos_timezone_offset;
-int dos_decimal_point;
-int dos_keyboard_layout;
+EMACS_INT dos_country_code;
+EMACS_INT dos_codepage;
+EMACS_INT dos_timezone_offset;
+EMACS_INT dos_decimal_point;
+EMACS_INT dos_keyboard_layout;
 unsigned char dos_country_info[DOS_COUNTRY_INFO];
 static unsigned char usa_country_info[DOS_COUNTRY_INFO] = {
   0, 0,				/* date format */
@@ -260,9 +261,9 @@ static unsigned char usa_country_info[DOS_COUNTRY_INFO] = {
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0	/* reserved */
 };
 
-int dos_hyper_key;
-int dos_super_key;
-int dos_keypad_mode;
+EMACS_INT dos_hyper_key;
+EMACS_INT dos_super_key;
+EMACS_INT dos_keypad_mode;
 
 Lisp_Object Vdos_version;
 Lisp_Object Vdos_display_scancodes;
@@ -284,7 +285,7 @@ restore_parent_vm_title (void)
   delay (50);
 }
 #endif /* !HAVE_X_WINDOWS */
-  
+
 void
 init_dosfns ()
 {
@@ -323,7 +324,7 @@ init_dosfns ()
   intdos (&regs, &regs);
   if (regs.x.cflag)
     /* Estimate code page from country code */
-    switch (dos_country_code) 
+    switch (dos_country_code)
       {
       case 45: /* Denmark */
       case 47: /* Norway */
@@ -401,6 +402,54 @@ init_dosfns ()
 }
 
 #ifndef HAVE_X_WINDOWS
+
+/* Emulation of some X window features from xfns.c and xfaces.c.  */
+
+/* Standard VGA colors, in the order of their standard numbering
+   in the default VGA palette.  */
+static char *vga_colors[16] = {
+  "black", "blue", "green", "cyan", "red", "magenta", "brown",
+  "lightgray", "darkgray", "lightblue", "lightgreen", "lightcyan",
+  "lightred", "lightmagenta", "yellow", "white"
+};
+
+/* Given a color name, return its index, or -1 if not found.  Note
+   that this only performs case-insensitive comparison against the
+   standard names.  For anything more sophisticated, like matching
+   "gray" with "grey" or translating X color names into their MSDOS
+   equivalents, call the Lisp function Qtty_color_desc (defined
+   on lisp/term/tty-colors.el).  */
+int
+msdos_stdcolor_idx (const char *name)
+{
+  int i;
+
+  for (i = 0; i < sizeof (vga_colors) / sizeof (vga_colors[0]); i++)
+    if (xstrcasecmp (name, vga_colors[i]) == 0)
+      return i;
+
+  return
+    strcmp (name, unspecified_fg) == 0 ? FACE_TTY_DEFAULT_FG_COLOR
+    : strcmp (name, unspecified_bg) == 0 ? FACE_TTY_DEFAULT_BG_COLOR
+    : FACE_TTY_DEFAULT_COLOR;
+}
+
+/* Given a color index, return its standard name.  */
+Lisp_Object
+msdos_stdcolor_name (int idx)
+{
+  extern Lisp_Object Qunspecified;
+
+  if (idx == FACE_TTY_DEFAULT_FG_COLOR)
+    return build_string (unspecified_fg);
+  else if (idx == FACE_TTY_DEFAULT_BG_COLOR)
+    return build_string (unspecified_bg);
+  else if (idx >= 0 && idx < sizeof (vga_colors) / sizeof (vga_colors[0]))
+    return build_string (vga_colors[idx]);
+  else
+    return Qunspecified;	/* meaning the default */
+}
+
 /* Support for features that are available when we run in a DOS box
    on MS-Windows.  */
 int
@@ -457,18 +506,191 @@ x_set_title (f, name)
   if (FRAME_MSDOS_P (f))
     {
       BLOCK_INPUT;
-      w95_set_virtual_machine_title (XSTRING (name)->data);
+      w95_set_virtual_machine_title (SDATA (name));
       UNBLOCK_INPUT;
     }
 }
 #endif /* !HAVE_X_WINDOWS */
 
+DEFUN ("file-system-info", Ffile_system_info, Sfile_system_info, 1, 1, 0,
+       doc: /* Return storage information about the file system FILENAME is on.
+Value is a list of floats (TOTAL FREE AVAIL), where TOTAL is the total
+storage of the file system, FREE is the free storage, and AVAIL is the
+storage available to a non-superuser.  All 3 numbers are in bytes.
+If the underlying system call fails, value is nil.  */)
+     (filename)
+     Lisp_Object filename;
+{
+  struct statfs stfs;
+  Lisp_Object encoded, value;
+
+  CHECK_STRING (filename);
+  filename = Fexpand_file_name (filename, Qnil);
+  encoded = ENCODE_FILE (filename);
+
+  if (statfs (SDATA (encoded), &stfs))
+    value = Qnil;
+  else
+    value = list3 (make_float ((double) stfs.f_bsize * stfs.f_blocks),
+		   make_float ((double) stfs.f_bsize * stfs.f_bfree),
+		   make_float ((double) stfs.f_bsize * stfs.f_bavail));
+
+  return value;
+}
+
+/* System depended enumeration of and access to system processes a-la
+   ps(1).  Here, we only return info about the running Emacs process.
+   (There are no other processes on DOS, right?)  */
+
+Lisp_Object
+list_system_processes ()
+{
+  Lisp_Object proclist = Qnil;
+
+  proclist = Fcons (make_fixnum_or_float (getpid ()), proclist);
+
+  return proclist;
+}
+
+Lisp_Object
+system_process_attributes (Lisp_Object pid)
+{
+  int proc_id;
+  Lisp_Object attrs = Qnil;
+
+  CHECK_NUMBER_OR_FLOAT (pid);
+  proc_id = FLOATP (pid) ? XFLOAT_DATA (pid) : XINT (pid);
+
+  if (proc_id == getpid ())
+    {
+      EMACS_INT uid, gid;
+      char *usr;
+      struct group *gr;
+      char cmd[FILENAME_MAX];
+      char *cmdline = NULL, *p, *q;
+      size_t cmdline_size = 0;
+      int i;
+      Lisp_Object cmd_str, decoded_cmd, tem;
+      double pmem;
+#ifndef SYSTEM_MALLOC
+      extern unsigned long ret_lim_data ();
+#endif
+
+      uid = getuid ();
+      attrs = Fcons (Fcons (Qeuid, make_fixnum_or_float (uid)), attrs);
+      usr = getlogin ();
+      if (usr)
+	attrs = Fcons (Fcons (Quser, build_string (usr)), attrs);
+      gid = getgid ();
+      attrs = Fcons (Fcons (Qegid, make_fixnum_or_float (gid)), attrs);
+      gr = getgrgid (gid);
+      if (gr)
+	attrs = Fcons (Fcons (Qgroup, build_string (gr->gr_name)), attrs);
+      strcpy (cmd, basename (__crt0_argv[0]));
+      /* Command name is encoded in locale-coding-system; decode it.  */
+      cmd_str = make_unibyte_string (cmd, strlen (cmd));
+      decoded_cmd = code_convert_string_norecord (cmd_str,
+						  Vlocale_coding_system, 0);
+      attrs = Fcons (Fcons (Qcomm, decoded_cmd), attrs);
+      /* Pretend we have 0 as PPID.  */
+      attrs = Fcons (Fcons (Qppid, make_number (0)), attrs);
+      attrs = Fcons (Fcons (Qpgrp, pid), attrs);
+      attrs = Fcons (Fcons (Qttname, build_string ("/dev/tty")), attrs);
+      /* We are never idle!  */
+      tem = Fget_internal_run_time ();
+      attrs = Fcons (Fcons (Qtime, tem), attrs);
+      attrs = Fcons (Fcons (Qthcount, make_number (1)), attrs);
+      attrs = Fcons (Fcons (Qstart,
+			    Fsymbol_value (intern ("before-init-time"))),
+		     attrs);
+      attrs = Fcons (Fcons (Qvsize,
+			    make_fixnum_or_float ((unsigned long)sbrk(0)/1024)),
+		     attrs);
+      attrs = Fcons (Fcons (Qetime, tem), attrs);
+#ifndef SYSTEM_MALLOC
+      /* ret_lim_data is on vm-limit.c, which is not compiled in under
+	 SYSTEM_MALLOC.  */
+      pmem = (double)((unsigned long) sbrk (0)) / ret_lim_data () * 100.0;
+      if (pmem > 100)
+#endif
+	pmem = 100;
+      attrs = Fcons (Fcons (Qpmem, make_float (pmem)), attrs);
+      /* Pass 1: Count how much storage we need.  */
+      for (i = 0; i < __crt0_argc; i++)
+	{
+	  cmdline_size += strlen (__crt0_argv[i]) + 1; /* +1 for blank delim */
+	  if (strpbrk (__crt0_argv[i], " \t\n\r\v\f"))
+	    {
+	      cmdline_size += 2;
+	      for (p = __crt0_argv[i]; *p; p++)
+		{
+		  if (*p == '"')
+		    cmdline_size++;
+		}
+	    }
+	}
+      /* Pass 2: Allocate storage and concatenate argv[].  */
+      cmdline = xmalloc (cmdline_size + 1);
+      for (i = 0, q = cmdline; i < __crt0_argc; i++)
+	{
+	  if (strpbrk (__crt0_argv[i], " \t\n\r\v\f"))
+	    {
+	      *q++ = '"';
+	      for (p = __crt0_argv[i]; *p; p++)
+		{
+		  if (*p == '\"')
+		    *q++ = '\\';
+		  *q++ = *p;
+		}
+	      *q++ = '"';
+	    }
+	  else
+	    {
+	      strcpy (q, __crt0_argv[i]);
+	      q += strlen (__crt0_argv[i]);
+	    }
+	  *q++ = ' ';
+	}
+      /* Remove the trailing blank.  */
+      if (q > cmdline)
+	q[-1] = '\0';
+
+      /* Command line is encoded in locale-coding-system; decode it.  */
+      cmd_str = make_unibyte_string (cmdline, strlen (cmdline));
+      decoded_cmd = code_convert_string_norecord (cmd_str,
+						  Vlocale_coding_system, 0);
+      xfree (cmdline);
+      attrs = Fcons (Fcons (Qargs, decoded_cmd), attrs);
+    }
+
+  return attrs;
+}
+
 void
 dos_cleanup (void)
 {
+  struct tty_display_info *tty;
+
 #ifndef HAVE_X_WINDOWS
   restore_parent_vm_title ();
 #endif
+  /* Make sure the termscript file is committed, in case we are
+     crashing and some vital info was written there.  */
+  if (FRAMEP (selected_frame))
+    {
+      struct frame *sf = XFRAME (selected_frame);
+
+      if (FRAME_LIVE_P (sf)
+	  && (FRAME_MSDOS_P (sf) || FRAME_TERMCAP_P (sf)))
+	{
+	  tty = CURTTY ();
+	  if (tty->termscript)
+	    {
+	      fflush (tty->termscript);
+	      fsync (fileno (tty->termscript));
+	    }
+	}
+    }
 }
 
 /*
@@ -484,89 +706,94 @@ syms_of_dosfns ()
   defsubr (&Smsdos_set_keyboard);
   defsubr (&Sinsert_startup_screen);
   defsubr (&Smsdos_mouse_disable);
+  defsubr (&Sfile_system_info);
 #ifndef HAVE_X_WINDOWS
   defsubr (&Smsdos_mouse_p);
-  Qmsdos_color_translate = intern ("msdos-color-translate");
-  staticpro (&Qmsdos_color_translate);
 #endif
 
   DEFVAR_INT ("dos-country-code", &dos_country_code,
-    "The country code returned by Dos when Emacs was started.\n\
-Usually this is the international telephone prefix.");
+	      doc: /* The country code returned by Dos when Emacs was started.
+Usually this is the international telephone prefix.  */);
 
   DEFVAR_INT ("dos-codepage", &dos_codepage,
-    "The codepage active when Emacs was started.\n\
-The following are known:\n\
-	437	United States\n\
-	850	Multilingual (Latin I)\n\
-	852	Slavic (Latin II)\n\
-	857	Turkish\n\
-	860	Portugal\n\
-	861	Iceland\n\
-	863	Canada (French)\n\
-	865	Norway/Denmark");
+	      doc: /* The codepage active when Emacs was started.
+The following are known:
+	437	United States
+	850	Multilingual (Latin I)
+	852	Slavic (Latin II)
+	857	Turkish
+	860	Portugal
+	861	Iceland
+	863	Canada (French)
+	865	Norway/Denmark  */);
 
   DEFVAR_INT ("dos-timezone-offset", &dos_timezone_offset,
-    "The current timezone offset to UTC in minutes.
-Implicitly modified when the TZ variable is changed.");
-  
+	      doc: /* The current timezone offset to UTC in minutes.
+Implicitly modified when the TZ variable is changed.  */);
+
   DEFVAR_LISP ("dos-version", &Vdos_version,
-    "The (MAJOR . MINOR) Dos version (subject to modification with setver).");
+	       doc: /* The (MAJOR . MINOR) Dos version (subject to modification with setver).  */);
 
 #ifndef HAVE_X_WINDOWS
   DEFVAR_LISP ("dos-windows-version", &Vdos_windows_version,
-    "The (MAJOR . MINOR) Windows version for DOS session on MS-Windows.");
+	       doc: /* The (MAJOR . MINOR) Windows version for DOS session on MS-Windows.  */);
 #endif
 
   DEFVAR_LISP ("dos-display-scancodes", &Vdos_display_scancodes,
-    "*When non-nil, the keyboard scan-codes are displayed at the bottom right\n\
-corner of the display (typically at the end of the mode line).\n\
-The output format is: scan code:char code*modifiers.");
+	       doc: /* *Controls whether DOS raw keyboard events are displayed as you type.
+When non-nil, the keyboard scan-codes are displayed at the bottom right
+corner of the display (typically at the end of the mode line).
+The output format is: scan code:char code*modifiers.  */);
+
   Vdos_display_scancodes = Qnil;
-  
+
   DEFVAR_INT ("dos-hyper-key", &dos_hyper_key,
-    "*If set to 1, use right ALT key as hyper key.\n\
-If set to 2, use right CTRL key as hyper key.");
+	      doc: /* *If set to 1, use right ALT key as hyper key.
+If set to 2, use right CTRL key as hyper key.  */);
   dos_hyper_key = 0;
-  
+
   DEFVAR_INT ("dos-super-key", &dos_super_key,
-    "*If set to 1, use right ALT key as super key.\n\
-If set to 2, use right CTRL key as super key.");
+	      doc: /* *If set to 1, use right ALT key as super key.
+If set to 2, use right CTRL key as super key.  */);
   dos_super_key = 0;
-  
+
   DEFVAR_INT ("dos-keypad-mode", &dos_keypad_mode,
-    "*Controls what key code is returned by a key in the numeric keypad.\n\
-The `numlock ON' action is only taken if no modifier keys are pressed.\n\
-The value is an integer constructed by adding the following bits together:\n\
- \n\
-  0x00	Digit key returns digit    (if numlock ON)\n\
-  0x01	Digit key returns kp-digit (if numlock ON)\n\
-  0x02	Digit key returns M-digit  (if numlock ON)\n\
-  0x03	Digit key returns edit key (if numlock ON)\n\
- \n\
-  0x00	Grey key returns char      (if numlock ON)\n\
-  0x04	Grey key returns kp-key    (if numlock ON)\n\
- \n\
-  0x00	Digit key returns digit    (if numlock OFF)\n\
-  0x10	Digit key returns kp-digit (if numlock OFF)\n\
-  0x20	Digit key returns M-digit  (if numlock OFF)\n\
-  0x30	Digit key returns edit key (if numlock OFF)\n\
- \n\
-  0x00	Grey key returns char      (if numlock OFF)\n\
-  0x40	Grey key returns kp-key    (if numlock OFF)\n\
- \n\
-  0x200	ALT-0..ALT-9 in top-row produces shifted codes.");
+	      doc: /* *Controls what key code is returned by a key in the numeric keypad.
+The `numlock ON' action is only taken if no modifier keys are pressed.
+The value is an integer constructed by adding the following bits together:
+
+  0x00	Digit key returns digit    (if numlock ON)
+  0x01	Digit key returns kp-digit (if numlock ON)
+  0x02	Digit key returns M-digit  (if numlock ON)
+  0x03	Digit key returns edit key (if numlock ON)
+
+  0x00	Grey key returns char      (if numlock ON)
+  0x04	Grey key returns kp-key    (if numlock ON)
+
+  0x00	Digit key returns digit    (if numlock OFF)
+  0x10	Digit key returns kp-digit (if numlock OFF)
+  0x20	Digit key returns M-digit  (if numlock OFF)
+  0x30	Digit key returns edit key (if numlock OFF)
+
+  0x00	Grey key returns char      (if numlock OFF)
+  0x40	Grey key returns kp-key    (if numlock OFF)
+
+  0x200	ALT-0..ALT-9 in top-row produces shifted codes.  */);
   dos_keypad_mode = 0x75;
-  
+
   DEFVAR_INT ("dos-keyboard-layout", &dos_keyboard_layout,
-    "Contains the country code for the current keyboard layout.\n\
-Use msdos-set-keyboard to select another keyboard layout.");
+	      doc: /* Contains the country code for the current keyboard layout.
+Use msdos-set-keyboard to select another keyboard layout.  */);
   dos_keyboard_layout = 1;	/* US */
-  
+
   DEFVAR_INT ("dos-decimal-point", &dos_decimal_point,
-    "If non-zero, it contains the character to be returned when the\n\
-decimal point key in the numeric keypad is pressed when Num Lock is on.\n\
-If zero, the decimal point key returns the country code specific value.");
+	      doc: /* The character to produce when kp-decimal key is pressed.
+If non-zero, this variable contains the character to be returned when the
+decimal point key in the numeric keypad is pressed when Num Lock is on.
+If zero, the decimal point key returns the country code specific value.  */);
   dos_decimal_point = 0;
 }
 #endif /* MSDOS */
+
+/* arch-tag: f5ea8847-a014-42c9-83f5-7738ad640b17
+   (do not change this comment) */
