@@ -1,6 +1,6 @@
 /* Machine description file for the alpha chip.
    Copyright (C) 1994, 1997, 1999, 2001, 2002, 2003, 2004, 2005, 2006,
-                 2007, 2008, 2009  Free Software Foundation, Inc.
+                 2007, 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 Author: Rainer Schoepf
 (according to authors.el)
@@ -95,7 +95,7 @@ NOTE-END
 
 #undef UNEXEC
 #define UNEXEC unexelf.o
-#ifndef GNU_LINUX
+#if !defined(GNU_LINUX) && !defined(__NetBSD__)
 #define DATA_START    0x140000000
 #endif
 
