@@ -1,6 +1,6 @@
 /* Machine description file for ARM-based non-RISCiX machines.
    Copyright (C) 1994, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-                 2008, 2009  Free Software Foundation, Inc.
+                 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -36,5 +36,11 @@ along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #define NO_REMAP
 
+/* armin76@gentoo.org reported that the lgcc_s flag is necessary to
+   build on ARM EABI under GNU/Linux (Bug#5518).  */
+#ifdef GNU_LINUX
+#define LIB_GCC -lgcc_s
+#endif
+
 /* arch-tag: 07856f0c-f0c8-4bd8-99af-0b7fa1e5ee42
    (do not change this comment) */
