@@ -1,6 +1,6 @@
 /* machine description file for the IA-64 architecture.
    Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
-                 2008, 2009, 2010, 2011  Free Software Foundation, Inc.
+                 2008, 2009, 2010, 2011, 2012  Free Software Foundation, Inc.
      Contributed by David Mosberger <davidm@hpl.hp.com>
 
 This file is part of GNU Emacs.
